
### Start of Variables ###

# Variables that start with "_" are template variables
# If found, the leading "_" will be removed.
# They are used to match "%%Variable%% template variables in 
# template pieces in the configuration files.

# _PageTitle is used in the Page Header template.
# i.e. <html><head><title>%%PageTitle%%</title>  etc.
_PageTitle = Installing Frogdown on a Raspberry Pi

# _PageID isn't used yet.  But it will be--make sure that every page has a unique _PageID.
_PageID = installing_Frogdown_RPi

# If using any php code...
#HTMLFileExt = .php

# Enable when you have a new Frogdown.zip
File = Frogdown.zip
File = GetFrogdown.sh.txt

# Force all images to be created? 
# (Usually only needed when you are fiddling with image sizes.)
# Or if you are experimenting with ImageMagickConfig in the config files
ForceImageCreation = 0

# Put assorted config variables as HTML comments at the start of the output file.
# Probably should only be used while debugging.  (Ctrl-U in browser)
Debug = 0

#Substitutions = frog|spring

# Also generate a word wrapped version of this file?
# Only is needed if you want to make this file available on the site.

### End of Variables ###

%##%
<!--
%id fii nl%
###Configuration for a Raspberry Pi
-->
%/##%

A Raspberry Pi is handy if you want to use Frogdown to see how it works.  Or for testing out new web sites before you make them live on a real server. 

%##%
A Raspberry Pi is handy if you want to use Frogdown to see how it works.  Or for testing out new web sites before you make them live on a real server. 
%/##%

%sidebar  sb=bqsidebar|t=Important note about the Raspberry Pi shortage%
The supply of Raspberry Pi machines is almost non-existant at the present time (spring 2022).  But low-end refurb PCs are still cheap and plentiful.  Discount Electronics has lots %l of cheap PC refurbs|https://discountelectronics.com/used-computers?page=1&sort=priceasc%.  Remove the slow spinner that has Win 10 and add a %l cheap SSD|https://www.bestbuy.com/site/pny-240gb-internal-sata-solid-state-drive/5900261.p?skuId=5900261% and install a Linux distro.   %l Debian|https://www.debian.org/% is what the Raspberry Pi OS is based on so if you use Debian you will be mostly trained for when Raspberry Pis are back in stock at the normal retail price.  This Raspberry Pi documentation is probably (mostly) applicable to Debian.
%/sidebar%

%%NameLinks%%

%id create-pi-boot-disk%
###Create a Raspberry Pi boot SD and configure it for running Frogdown

Frogdown will run on any Raspberry Pi.  If you are starting out with a new Raspberry Pi, %l here is how to create and configure a boot SD|PiRef.html#sdcard% that works.

The assumption from here on out is that you are running from a command line.  

%id ssh-and-winscp %
###SSH terminal with PuTTY and WinSCP for copying files

Use %l PuTTY for getting a terminal window from your Windows machine and use WinCSP for copying files between your local machine and your Raspberry Pi|PiRef.html#ssh%

<!-- You can use %l SSH|PiRef.html#ssh% to create terminal windows that will work on your Windows (or Mac) desktop/laptop machine. -->

%id perl ht = The language that Frogdown is written in%
###Perl (and needed CPAN modules)

Frogdown is written in Perl.  Perl is probably already installed.  Check by typing

	perl --version
	
But you will probably need to %l install some Perl modules from CPAN|PiRef.html#perl-modules%

%id imagemagick ht = For scripted image resizing%
###ImageMagick is needed by Frogdown to deal with images

Frogdown needs %l ImageMagick|https://imagemagick.org/index.php% to convert image files.
	
	sudo apt update
	sudo apt install imagemagick



%id environment ht = So Frogdown can find itself%
###Setting Environment variables
	
Edit ~/.bashrc (`nano ~/.bashrc`) and copy this

	# Frogdown starts here
	
	# alias is so you can process a single file like 'fd index.md frog`
	alias fd='RunFrogdown.sh '

	# So Frogdown can find /etc/markcms config files
	export FHOME=$HOME/Frogdown

	# Add the Frogdown bin directory to the existing path.
	export PATH=$PATH:$HOME/Frogdown/bin

	# Set PERL5LIB so that Frogdown can find the Frogdown Perl modules.
	export PERL5LIB=$HOME/Frogdown/bin/perl5lib

	# Create new custom commands for changing into directories
	# we always need to change to whenever we log in.
	
	# Change to the directory where we edit Frogdown .md files
	cdfa ()
	{
		cd "/home/pi/Frogdown/frog_authoring/Frogdown"
	}

	# Change to the directory where the .html files are written
	cdfh ()
	{
		# If you are using Apache2's default DocumentRoot
		#cd "/var/www/html/Frogdown"
		# If you haven't installed a web server.
		# Or have changed the web server's DocumentRoot
		cd "/home/pi/html/frog_html/Frogdown"
	}

to the end of `.bashrc`.  Save it (`Ctl-o`). 

%id chickenegg%
###Resolving a chicken and egg problem

(There is a chicken-egg problem here.  You need to have downloaded and unzipped `Frogdown.zip` before you can run `TestPerl.pl`.  To unravel the time twist, go to the %l download-and-unzip|#download% section.  Once you have Frogdown downloaded and unzipped, come back here to test.

%b%Log in again in a different session before closing %tt%.bashrc%/tt%.%/b%  Type `TestPerl.pl`.  It *should* run if the path variable was updated to include `~/Frogdown/bin`  (`echo $PATH` to see the `PATH` variable.)  If TestPerl runs, it should display

	TestPerl.pl  1.02  2022-04-11
	Tests to see if you are set up to process Frogscript with MarkdownWrapper.pl
	'FHOME' is used in place of 'HOME'

	This is good:
	  Module 'FrogUtils::utility' is loaded.
	  Module 'FrogUtils::MarkdownW.pm' is loaded.
	  Module 'FrogUtils::SmartyPantsW' is loaded.
	  Module 'File::Path' is loaded.
	  Module 'Digest::MD5' is loaded.
	  Module 'Date::Calc' is loaded.

	  Your HOME directory is '/home/pi/Frogdown/'
	  Your configuration directory (/home/pi/Frogdown/etc/markcms/) exists.
	  'frog.cfg' exists in your configuration directory.
	  'frog-site.cfg' exists in your configuration directory.


	Testing to see if ImageMagick is installed
	Good! ImageMagick (convert) is installed. Which is close enough.

The `FrogUtils` modules are the ones we supply.  The rest of the modules are from CPAN and %l may need to be installed|PiRef.html#perl-modules% if `TestPerl.pl` complains.

%#%
%sidebar  sb=bqsidebar|t=Installing CPAN modules%

Use %l cpanm|https://metacpan.org/dist/App-cpanminus/view/bin/cpanm% to install CPAN modules:

	sudo apt update
    sudo apt install curl
	curl --version
    sudo curl -L http://cpanmin.us | perl - --sudo App::cpanminus
	cpanm --version | less
	
[https://forums.raspberrypi.com/viewtopic.php?t=13410#](https://forums.raspberrypi.com/viewtopic.php?t=13410#)

then install CPAN perl modules like

    sudo cpanm Digest::MD5
	sudo cpanm Date::Calc

(It takes time to install CPAN modules this way.  Minutes...)
%/sidebar%
%/#%

%id download%
###Downloading and unzipping Frogdown

*If* you have followed all my instructions on setting Apache's (or whatever web server) `DocumentRoot`, then you can use this %l 3 second install|GetFrogdown.sh.txt%.  Download it and save it as `GetFrogdown.sh` and put it in your `~` (home) directory (i.e., `/home/pi`).  `chmod +x GetFrogdown.sh` to make it executable.  

%#%
	#!/bin/bash
	#

	echo Downloads and installs Frogdown from scratch

	# wget doesn't do overwriting very well.
	rm Frogdown.zip
	wget "http://www.frogymandias.org/Frogdown/Frogdown.zip"
	unzip -q Frogdown.zip

	# Make the executable files be executable
	chmod +x ~/Frogdown/bin/*.sh
	chmod +x ~/Frogdown/bin/*.pl
	chmod +x ~/Frogdown/frog_authoring/Frogdown/*.sh
	chmod +x ~/Frogdown/LinuxOnly/InitialTests/*.sh
	
	# Only for the initial install!  Then comment this line out!
	cp -f ~/Frogdown/etc/markcms-linux-raspberry-pi/*.cfg ~/Frogdown/etc/markcms
%/#%	

And then jump to checking the %l configuration files|#configfiles%

*However* if you aren't using all my defaults (for setting up Apache, etc.), then it is better to install a step at a time.
	
Download %l Frogdown.zip|Frogdown.zip% into your home directory ~ (`/home/pi` usually). 

	cd ~
	wget "http://www.frogymandias.org/Frogdown/Frogdown.zip"
	unzip -q Frogdown.zip

Do some initial configuration
	
	cd Frogdown/etc/markcms-linux-raspberry-pi
	chmod 744 first-time.sh
	./first-time.sh
	cd ../markcms

`first-time.sh` Makes all the `.sh` and `.pl` files be executable (they aren't after being unzipped). Then it copies the configuration files from `Frogdown/etc/markcms-linux-raspberry-pi` to `Frogdown/etc/markcms`.  You should then `cd ../markcms` so you can edit the configuration files.  

%b%You only run `first-time.sh` the first time you download and install Frogdown.%/b% If you update to a new version by downloading and unzipping Frogdown, do *not* run `first-time.sh` again.  Because it will overwrite your configuration files in `~/etc/Frogdown/etc/markcms`
	
At this point it is probably best to open another ssh terminal so you can edit the config files in `markcms` while running scripts and such in the other terminal window.

%id configfiles ht = So Frogdown will know what to do%
###Configuration files

Edit `~/Frogdown/etc/markcms/frog-site.cfg`  There are two variables that need to be checked

	# .md files you edit are here
	BaseInDir = ~/Frogdown/frog_authoring
	
	# The converted .html files will be written here
	# Assuming that you changed Apache2's DocumentRoot
	# (Or haven't installed any web server)
	BaseOutDir = ~/Frogdown/frog_html
	# Apache default (which is owned by root.  chown -R...)
	#BaseOutDir = /var/www/html

If you already have a web server running and its `DocumentRoot` points to a different directory, then change `BaseOutDir` to be the same directory.  i.e., where the .html files are to be written.  

If you don't have a web server installed then leave `BaseOutDir = ~/Frogdown/frog_html` <!-- and use X Windows file manager to navigate to `~/Frogdown/frog_html/Frogdown` and click on `index.html` It should open in your default browser and will work.  You don't really need a web server to test Frogdown on your own machine. -->
	
`~/Frogdown/etc/markcms/frog.cfg` has variables that describe various defaults.  You don't need to editing anything here now.  Wait until you are more familiar with Frogdown and then come back to `frog.cfg`.

`frog-site-locations.cfg` isn't used yet.  It is there for some future use.

(If this is the first time you installed Frogdown on this machine, go back to the %l chicken-egg|#chickenegg% part to run some tests.  Then come back here when all the tests have passed.)

%id runfrogdowndebug%
###Running Frogdown in debug mode to test first

`cd ~/Frogdown/frog-authoring/Frogdown`, (`cdfa` if you added the %l `.bashrc` functions|#environment%.) if needed then

`RunFrogdown.sh index.md debug | less`

This tells `MarkdownWrapper.pl`, which is the Frogdown converter to process `index.html`, but with the 'debug' option.  With debug, it won't generate a .html file.  Instead, it will display a lot of configuration information and then stop.  You should see something like

%##%

	Doing 'Markdown.pl frog index.md debug  '

	MarkdownWrapper 0.44  2022-04-01
	Read config file /home/pi/Frogdown/etc/markcms/frog.cfg
	Read config file /home/pi/Frogdown/etc/markcms/frog-site.cfg
	Read config file /home/pi/Frogdown/etc/markcms/frog-site-locations.cfg
	Changing '~' to '/home/pi/' in '~/Frogdown/frog_authoring'
	  is now '/home/pi/Frogdown/frog_authoring'
	Changing '~' to '/home/pi/' in '~/Frogdown/frog_html'
	  is now '/home/pi/Frogdown/frog_html'
	Read input file: 'index.md'

	Basename:          'index'
	Ext:               'md'
	Filename:          'index.md'
	htmlfile           'index.html'
	Config             'frog.cfg'

	HomePath:          '/home/pi/'
	ConfigPath         '/home/pi/Frogdown/etc/markcms/'
	Path (pwd):        '/home/pi/Frogdown/frog_authoring/Frogdown/'
	real pwd:          '/home/pi/Frogdown/frog_authoring/Frogdown/'
	JustDir:           'Frogdown/'
	%%PathToHome%%:    '../'

%/##%

    (snip...)
	
	BaseInDir:         '/home/pi/Frogdown/frog_authoring/'
	BaseOutDir:        '/home/pi/Frogdown/frog_html/'
	InDir:             '/home/pi/Frogdown/frog_authoring/Frogdown/
	OutDir:            '/home/pi/Frogdown/frog_html/Frogdown/
	FilestoBeCopied:   'frog.csst'

<!--
	(Linux)
	Debug mode--did not create any files.

	 index.md debug | less
-->

`InDir` and `Outdir` should correspond to the variables you set in `frog-site.cfg`  If this is correct then...

%id runfrogdown%
###Running Frogdown for real to generate your web site.

Type

`RunFrogdown.sh index.md`

Will run `MarkdownWrapper.pl`.  It will display a number of things, but the last line should be

`Created HTML file: /home/pi/Frogdown/frog_html/Frogdown/index.html`

Find the IP address of your Pi (`hostname -I`) and put that IP in your browser's URL bar.  Add `/Frogdown` after the URL (the IP) and press enter.  You *should* see the Frogdown page, that should look just like %l http://www.frogymandias.org/Frogdown/|http://www.frogymandias.org/Frogdown/% (plus or minus a few changes.)  At this point the rest of the pages won't exist, so clicking on the links at the bottom of the page will give a "404 File not found" error.  So let's fix this and generate the rest of the pages.  Type
	
	./DoAllFrogs.sh

Which will generate `GenAll.sh` which is a script that will run Frogdown (`MarkdownWrapper.pl`) on all the .md files in the current directory.  (Run `./GenAll.sh`) Now you can click on the links at the bottom of the index page and pages should exist.  You can go to `~/Frogdown/frog_html/Frogdown` (or wherever `OutDir` is) and examine the .html files (and image files) that were just created.

If so, then congratulations!  You now have a working Frogdown running on your Pi.  See %l DogFood.html|DogFood.html% for instructions on the Frogdown markup language.  (%l DogFood.text|DogFood.text% for the Frogdown source code for that page.)

You can now use Frogdown to create your own web pages.  Make sure to copy `DoAllFrogs.sh` and `frog.csst` to any directory you will be working in.  Look at the .md files in the Frogdown directory as example.  Whenever I start a new page, I copy the configuration variables at the top of an existing .md file to a new file and then start writing text formated with Frogdown markup.

%id apache ht = The web server%
###Optional: install Apache (and PHP)

%sidebar  sb=bqsidebar|t=A note about web servers%
It is real handy to have a web server running on your Raspberry Pi.   This documentation assumes that you install Apache as is documented here and that `DocumentRoot` points to `/var/www/html`  If you already have a different web server running, then you will need edit `~/Frogdown/etc/markcms/Frog-site.cfg` and change `BaseOutDir\ =\ /var/www/html` to point to where your web server's `DocumentRoot` is.  

Also note: all the Frogdown sample files will be put in a `Frogdown` directory that is *above* `DocumentRoot` so none of your existing `DocumentRoot` files will (should?) be overwritten.  And %l jump past|#download% the following line about installing Apache.
%/sidebar%

If you don't install a web server, you can see how the generated .html files look by navigating to the directory where the .html files are (default of `~/Frogdown/frog_html) and double clicking on `index.html` If you have a Samba share working on your ~ directory, you can navigate there from your Windows machine.

Or you might went to install the Apache web server which might not be installed on a fresh Raspberry Pi. Then you can access your .html files from any machine on your subnet. (The IP of your Raspberry Pi is the URL.)   To %l install Apache (and php)|PiRef.html#install-apache-php%:

%id final-hints%
###Final hints:

Because I usually only am editing one or two files at a time I make a copy of GenAll.sh and modify it to only process the files I am editing.  

	cp -p GenAll.sh GenOne
	nano GenOne
	
and put "#" at the beginning of the lines you aren't currently editing.

And/or rename `~/Frogdown/bin/RunFrogdown.sh` to have a shorter name.  Or add an alias to ~./bashrc like

	alias fd='RunFrogdown.sh '
	
log in again, change to the directory that contains your .md files and you can type (for example) `fd index.md` to convert `index.md` to `index.html`.  You are generally only working on one or two pages in the same session and `fd index.md` (etc.) is easy to type.  This is (roughly) how I've created all the pages on %l http://www.frogymandias.org/|http://www.frogymandias.org%

%id problems ht = Some things to check%
###Uh oh, it doesn't work!

If things aren't working correctly, run `RunFrogdown.sh index.md debug | less` again and examine the the variables closely.

Look at `InDir` and `OutDir` especially.  Look through the configuration files in `~/Frogdown/etc/markcms`.  And/or double check the changes to `.bashrc`  And/or any of the above instructions.
