### Start of Variables ###

## Version that lives in Frogdown directory

# Variables that start with "_" are template variables
# If found, the leading "_" will be removed.
# They are used to match "%%Variable%% template variables in 
# template pieces in the configuration files.

# _PageTitle is used in the Page Header template.
# i.e. <html><head><title>%%PageTitle%%</title>  etc.
_PageTitle = Some PHP code

# _PageID isn't used yet.  But it will be--make sure that every page has a unique _PageID.
_PageID = Some_PHP

# Force all images to be created? 
# (Usually only needed when you are fiddling with image sizes.)
# Or if you are experimenting with ImageMagickConfig in the config files
ForceImageCreation = 0

# Put assorted config variables as comments at the start of the output file.
# Probably should only be used while debugging.
Debug = 0

# Also generate a word wrapped version of this file?
# Only is needed if you want to make this file available on the site.
# MakeTextFile = 1

# If using any PHP code, we will want the file extension to be 'php'
# not 'html'  So...
HTMLFileExt = .php

### End of Variables ###

(This page `SomePHP.md` was last edited on %attr SomePHP.md|dt%)

This is a demo of a few bits of embedded PHP.  Do `Ctl-U` to view source code and try to find any .php code. (Hint, there isn't because PHP code is executed on the web server before it gets to you.)  

But look at the Frogdown source code: %l SomePHP.text|SomePHP.text% (which is really a copy of `SomePHP.md`) to see the embedded PHP code.  You can also examine `SomePHP.php` with nano in your web server's `DocumentRoot` directory.  At this point the web server hasn't executed the PHP code.

First a quick validity test that PHP works.

%save%<?php

	// Frogdown or Markdown tags don't exist in a 'save' zone.
	// Or shouldn't.  Don't try to use any Frog/Markdown tags.
	
	print "<p>This was printed by PHP.  ";
			
	$Filename = "Frogdown.zip";	 // for a bit later
								 // Variables are better than hard-wired data.
	print "Still in PHP.  The PHP code defined a PHP <tt>\$Filename</tt> variable that is defined as: <tt>$Filename</tt></p>\n";							 

?>%/save%

Now we are out of PHP and back in regular HTML.  Now we'll use a PHP function to find when <?php print "<tt>$Filename</tt>"; ?> was last modified.  (Actually the preceding sentence went into PHP to print the value of the `$Filename` variable.  Which isn't a variable in regular HTML.  You don't need to break out `\%save\%` for short inline PHP code.  Only for longer multi-like PHP that has blank lines.)

%save%<?php

	// Can't use Frogdown or Markdown markup tags in a 'save' block.
	print "<p><tt>$Filename</tt> was last modified on " . file_mod_date($Filename,7) . "</p>\n";
	print "<p>This page (<tt>SomePHP.php</tt>) was last modified (generated) on " . file_mod_date("SomePHP.php",7) . "</p>\n";
?>%/save%

`SomePHP.php` was *generated* by Frogdown so it will (probably) have a different modification date than `SomePHP.md` (which was modified (edited) on %attr SomePHP.md|dt%))	

Then we'll use PHP to find out how large <?php print "<tt>$Filename</tt>"; ?> is.

%save%<?php

	print "<p><tt>$Filename</tt> is " . file_size($Filename) . " long.</p>\n";


?>%/save%

And we are back to regular HTML.

%save%
<?php

// Utility functions for determining info about a file

// Returns a file's modification time
function file_mod_date($fname, $newtime = 7) {	 // $newtime is how many days are considered as being new.

	// Number of days to flag as "New!"
	if ($newtime) {
		$new = $newtime;
	} else {
		$new = "7";
	}

	$mtime = "Error determining file creation time.";
	//$mtime = HTMLDIR;
	if (file_exists($fname)) { 
		// http://www.php.net/manual/en/function.date.php
	        clearstatcache();
	        $mtime = date("F d, Y h:i a T", filemtime($fname));

		$ftime = date("Y-m-d", filemtime($fname));
		$before = mktime (0,0,0,date("m"),date("d")-$new,  date("Y"));
		$btime = date("Y-m-d", $before);
		$elaptime = $before = $ftime;
		if ($btime < $ftime) {
		  $new = "<b><font color=\"#ff0000\"><i>New!</i></font></b> ";
		  $mtime = $new . " " . $mtime;
		  //$mtime .= "  (" . date("d")-$new . " days old)";
		}
		// $mtime .= " (" . file_size($fname) . " bytes)";
	}
	return $mtime;
}


// Returns a file's size
function file_size($fname) {
	$decade="";
	$size = "Can't determine file size.";
	//$size = $fname;
	if (file_exists($fname)) { 
		$size = filesize($fname);
		if ($size >= 1048576) {
			$size = number_format( ($size/1048576) , 1 ) . " megabytes";
		} else if ($size >= 1024 ) {
			$size = number_format( ($size/1024) , 0 ) . " kilobytes";
		} else {
			$size = number_format($size) . " bytes";
		}
	}
	return $size;
}
 ?>
 %/save%
 